@@ -1,7 +1,6 @@
 from lark import Lark, Tree, Token
 from lark.exceptions import UnexpectedInput, UnexpectedEOF
 from pathlib import Path
-import importlib.resources
 
 import sys
 import json
@@ -208,12 +207,9 @@ def load_package(name):
     print(f"Importing functions from package: {name}")
     try:
         # import the .lib file which contains the start typing for the associated python file
-        if RUN_LOCAL:
-            with open("lib/" + name + ".lib", "r") as file:
-                package = file.read()
-        else:
-            with importlib.resources.open_text("start_compiler.lib", f"{name}.lib") as f:
-                package = f.read()
+        with open("lib/" + name + ".lib", "r") as file:
+            package = file.read()
+
         imported_functions = json.loads(package)
 
         # add the lib to the defined functions
@@ -223,10 +219,7 @@ def load_package(name):
             symbols[f] = imported_functions[f]["return_type"]
 
         # import the python file
-        if RUN_LOCAL:
-            imports.append("import lib." + name + " as " + name)
-        else:
-            imports.append("import start_compiler.lib." + name + " as " + name)
+        imports.append("import lib." + name + " as " + name)
     except Exception as _:
         raise StartError("ERROR loading imported package " + "lib/" + name + ".lib")
     return None
@@ -301,8 +294,8 @@ def to_python(t, state, indent, scope, namespace):
         if compilerPass == 0:
             function = namespace + val(t, "name")
             returnType = val(t, "type")
-            if returnType is None:  # if the return type is not specified we assume the function does not return anything so its a void return type
-                returnType = "void"
+            #if returnType is None:  # if the return type is not specified we assume the function does not return anything so its a void return type
+            #    returnType = "void"
 
             found_return = False  # flag so we know if we found a return value or not later in teh oarse Tree (bit ugly, to use global flag for this)
             if buildingSymbolTable:
@@ -323,7 +316,7 @@ def to_python(t, state, indent, scope, namespace):
                     result += to_python(cmd, str(arg_count), indent + "\t", "function", function + ".")  # set the scope to function
                     arg_count += 1
                 elif not buildingSymbolTable:
-                    result += to_python(cmd, returnType, indent + "\t", "function", function + ".")  # set the scope to function and pass the return type as state so that the return statement can check for the correct type
+                    result += to_python(cmd, state, indent + "\t", "function", function + ".")  # set the scope to function
 
             if not found_return and not buildingSymbolTable:
                 if returnType == "void":
@@ -332,19 +325,21 @@ def to_python(t, state, indent, scope, namespace):
                     raise StartError(f"ERROR in line {line}: Function <{function}> misses return statement and should return type <{returnType}>.")
 
     elif t.data == "return":
+        if namespace == "":
+            raise StartError(f"ERROR in line {line}: Found a return statement outside of a function.")
         return_expression = to_python(get(t, "return_expression"), state, indent, scope, namespace)
-        if return_expression != "" and state != "void":  # This means we found a return expression and we need one so parse it and check types
+        if return_expression != "" and returnType != "void":  # This means we found a return expression and we need one so parse it and check types
             return_symbol = to_symbol(get(t, "return_expression"), state, "", scope, namespace)
-            if symbol_type(line, return_symbol) != state:
+            if symbol_type(line, return_symbol) != returnType:
                 raise StartError(f"ERROR in line {line}: Type mismatch in function <{namespace[0:-1]}>, should return type <{returnType}> but instead returns <{symbol_type(line, return_symbol)}>.")
             else:
                 result = f"\n{indent}return {return_expression}"
                 found_return = True
 
-        elif return_expression == "" and state != "void":  # This means we found no expression but we do need one, so raise an error
+        elif return_expression == "" and returnType != "void":  # This means we found no expression but we do need one, so raise an error
             raise StartError(f"ERROR in line {line}: Function <{namespace[0:-1]}>, should return type <{returnType}> but instead returns nothing.")
-        elif return_expression != "" and state == "void":  # This means we found an expresswion but we dont need one, so raise an error
-            raise StartError(f"ERROR in line {line}: Function <{namespace[0:-1]}> should not return a value as it is defined without a return type.")
+        elif return_expression != "" and returnType == "void":  # This means we found an expresswion but we dont need one, so raise an error
+            raise StartError(f"ERROR in line {line}: Function <{namespace[0:-1]}> should not return a value as it is defined as void (no return).")
         else:  # we found only a return and we need that
             result = f"\n{indent}return"
             found_return = True
@@ -393,9 +388,9 @@ def to_python(t, state, indent, scope, namespace):
         left_hand_side = setter(to_python(get(t, "variable"), state, "", scope, namespace))
         if val(t, "assign_operator") == "=":
             result = f"{left_hand_side} = {right_hand_side}.clone()"
-        if val(t, "assign_operator") == "->":
+        elif val(t, "assign_operator") == "->":
             result = f"{left_hand_side} = {right_hand_side}"
-        if val(t, "assign_operator") == ":=":
+        elif val(t, "assign_operator") == ":=":
             result = f"{right_hand_side}.copy({left_hand_side})"
 
         left_symbol = to_symbol(get(t, "variable"), state, "", scope, namespace)
@@ -450,6 +445,8 @@ def to_python(t, state, indent, scope, namespace):
             # parse the variable (which checks if it is defined)
             # now retreive the proper function name using the variable"s type (which must be defined because every variable needs a type)
             function = val(t, "type") + "." + function
+            if function not in functions:
+                raise StartError(f"ERROR in line {line}: Function <{symbol}> not defined.")
             # add the function symbol name for checking
             functions_to_check[function] = line
 
@@ -461,9 +458,10 @@ def to_python(t, state, indent, scope, namespace):
             if function in functions:  # we know the function, so use the function name (required for imported modules and built in operators etc)
                 result = f"{functions[function]['name']}({parse_arguments(line, t, state, '', scope, namespace, function)})"
             else:
+                raise StartError(f"ERROR in line {line}: Function <{symbol}> not defined.")
                 # add the function symbol name for checking, and use the function as is as it must be defined in the code later
-                functions_to_check[function] = line
-                result = f"{function}({parse_arguments(line, t, state, '', scope, namespace, function)})"
+                #functions_to_check[function] = line
+                #result = f"{function}({parse_arguments(line, t, state, '', scope, namespace, function)})"
 
     elif t.data == "constant":  # just return the token value
         if get(t, "number") is not None:
@@ -489,10 +487,11 @@ def to_python(t, state, indent, scope, namespace):
         if scope == "":
             parsing_event = True
             block = to_python(get(t, 'block'), state, '\t\t', scope, namespace)
+            condition = to_python(get(t, 'expression'), state, '', scope, namespace)
             event_code += (f"\ndef event{event_count}():"
-                           f"\n\tif {to_python(get(t, 'expression'), state, '', scope, namespace)}: {block}\n")
+                           f"\n\tif {condition}: {block}\n")
             parsing_event = False
-            result = f"\n{indent}start_events[event{event_count}] = event{event_count}"
+            result = f'\n{indent}start_events["""{condition}"""] = event{event_count}'
             event_count += 1
         else:
             raise StartError(f"ERROR in line {line}: when blocks can only be defined in global scope (events are global).")
@@ -501,7 +500,7 @@ def to_python(t, state, indent, scope, namespace):
         if state == "while":
             result = "\n" + indent + "break"
         else:
-            raise StartError("ERROR in line {line}: break outside a while loop.")
+            raise StartError(f"ERROR in line {line}: break outside a while loop.")
     elif t.data == "exit":
         result = "\n" + indent + "listener.stop()"
         result += "\n" + indent + "exit(0)"
@@ -536,23 +535,25 @@ def compile_start(parse_tree):
         result += to_python(inst, state="", indent="\t", scope="", namespace="")
 
     # add pass for an empty file
-    if len(result.split("\n")) <= 2:
+    if len(result.split("try:\n")) <= 1:
         result += "\n\tpass"
-    result += "\nexcept Exception as e:\n\tprint('Start runtime error in line '+str(StartError.lineNumber)+': ',e)"
+    result += "\nexcept Exception as e:\n\tprint(f'Start runtime error in line {StartError.lineNumber}: {e}')"
     return result + "\nfinally:\n\tlistener.stop()\n"
 
 def compile_source(file_name):
-    with open(file_name, "r") as file:
-        # Read the entire contents of the file into a string
-        source = file.readlines()
-    # check if there are imports
-    for line in source:
-        if line.strip().startswith("include"):
-            include_file = line.strip()[7:].strip() + ".start"
-            print("Including file:" + include_file)
-            # Add the include sourcefile to this source recursively
-            source = compile_source(include_file) + source
-
+    try:
+        with open(file_name, "r") as file:
+            # Read the entire contents of the file into a string
+            source = file.readlines()
+        # check if there are imports
+        for line in source:
+            if line.strip().startswith("include"):
+                include_file = line.strip()[7:].strip() + ".start"
+                print("Including file:" + include_file)
+                # Add the include sourcefile to this source recursively
+                source = compile_source(include_file) + source
+    except:
+        raise StartError(f"ERROR opening start source file")
     return source + ["\n"]  # make sure that every file at least ends with an enter
 
 def main(file_name):
@@ -561,14 +562,15 @@ def main(file_name):
             with open(Path(__file__).parent / Path('start_grammar.ebnf'), 'r') as file:
                 # Read the entire contents of the file into a string
                 grammar = file.read()
-            source = "".join(compile_source(file_name))
 
         except Exception as e:
             print(e)
-            raise StartError(f"ERROR opening grammar <start_grammar.ebnf> or source file <{file_name}>")
+            raise StartError(f"ERROR opening grammar <start_grammar.ebnf>")
+
+        source = "".join(compile_source(file_name))
 
         try:
-            parser = Lark(grammar, parser="earley", propagate_positions=True)
+            parser = Lark(grammar, propagate_positions=True)  # The parser matters for ambiguity and there are no guarentees
             parse_tree = parser.parse(source)
         except UnexpectedEOF as _:
             raise StartError("Syntax Error end missing in start-end block")
